@@ -183,6 +183,19 @@ EXTERN int
 iscsi_set_initial_r2t(struct iscsi_context *iscsi, enum iscsi_initial_r2t initial_r2t);
 
 
+enum iscsi_chap_auth {
+	ISCSI_CHAP_MD5 = 5,
+	ISCSI_CHAP_SHA_1 = 6,
+	ISCSI_CHAP_SHA_256 = 7,
+	ISCSI_CHAP_SHA3_256 = 8,
+};
+
+EXTERN enum iscsi_chap_auth
+iscsi_get_auth(struct iscsi_context *iscsi);
+
+EXTERN void
+iscsi_set_auth(struct iscsi_context *iscsi, enum iscsi_chap_auth auth);
+        
 /*
  * This function is used to parse an iSCSI URL into a iscsi_url structure.
  * iSCSI URL format :
@@ -335,6 +348,29 @@ enum iscsi_header_digest {
 EXTERN int iscsi_set_header_digest(struct iscsi_context *iscsi,
 			    enum iscsi_header_digest header_digest);
 
+/*
+ * Types of data digest we support. Default is NONE
+ */
+enum iscsi_data_digest {
+	ISCSI_DATA_DIGEST_NONE        = 0,
+	ISCSI_DATA_DIGEST_NONE_CRC32C = 1,
+	ISCSI_DATA_DIGEST_CRC32C_NONE = 2,
+	ISCSI_DATA_DIGEST_CRC32C      = 3,
+	ISCSI_DATA_DIGEST_LAST        = ISCSI_DATA_DIGEST_CRC32C
+};
+
+/*
+ * Set the desired data digest for a scsi context.
+ * Data digest can only be set/changed before the context
+ * is logged in to the target.
+ *
+ * Returns:
+ *  0: success
+ * <0: error
+ */
+EXTERN int iscsi_set_data_digest(struct iscsi_context *iscsi,
+			    enum iscsi_data_digest data_digest);
+
 /*
  * Specify the username and password to use for chap authentication
  */
@@ -492,6 +528,38 @@ EXTERN int iscsi_reconnect(struct iscsi_context *iscsi);
  */
 EXTERN int iscsi_reconnect_sync(struct iscsi_context *iscsi);
 
+/*
+ * Disconnect a connection to a target and try to reconnect (async version).
+ * This call returns immediately and the reconnect is processed in the
+ * background. Commands send to this connection will be queued and not
+ * processed until we have successfully reconnected.
+ *
+ * This will re-start connection (to the configured original portal) even if a
+ * pending reconnection is already underway, which may be useful if the current
+ * connection is not progressing.  It does not over-ride any existing re-try
+ * backoff or max retries state.
+ *
+ * Returns:
+ *  0 reconnect was successful
+ * <0 error
+ */
+EXTERN int iscsi_force_reconnect(struct iscsi_context *iscsi);
+
+/*
+ * Disconnect a connection to a target and try to force reconnection (sync
+ * version). This call will block until the connection is reestablished.
+ *
+ * Returns:
+ *  0 reconnect was successful
+ * <0 error
+ */
+EXTERN int iscsi_force_reconnect_sync(struct iscsi_context *iscsi);
+
+/*
+ * Reset iscsi auto reconnect next_reconnect time to the current time.
+ */
+EXTERN void iscsi_reset_next_reconnect(struct iscsi_context *iscsi);
+
 /*
  * Asynchronous call to perform an ISCSI login.
  *
@@ -549,12 +617,12 @@ EXTERN int iscsi_logout_sync(struct iscsi_context *iscsi);
 
 struct iscsi_target_portal {
        struct iscsi_target_portal *next;
-       const char *portal;
+       char *portal;
 };
 
 struct iscsi_discovery_address {
        struct iscsi_discovery_address *next;
-       const char *target_name;
+       char *target_name;
        struct iscsi_target_portal *portals;
 };
 
@@ -1510,6 +1578,13 @@ EXTERN void scsi_task_set_iov_in(struct scsi_task *task, struct scsi_iovec *iov,
 
 EXTERN int scsi_task_get_status(struct scsi_task *task, struct scsi_sense *sense);
 
+/*
+ * This function returns 1 if the task is queued for send (for example, the task has been created
+ * with an asynchronous request and has not been put on the wire right away due to the socket
+ * or the ISCSI session state); otherwise returns 0.
+ */
+EXTERN int iscsi_scsi_is_task_in_outqueue(struct iscsi_context *iscsi, struct scsi_task *task);
+
 /*
  * This function is used when you want to cancel a scsi task.
  * The callback for the task will immediately be invoked with SCSI_STATUS_CANCELLED.
@@ -1621,6 +1696,12 @@ iscsi_set_reconnect_max_retries(struct iscsi_context *iscsi, int count);
 EXTERN void
 iscsi_set_no_ua_on_reconnect(struct iscsi_context *iscsi, int state);
 
+/* Set callback on iscsi file descriptor on duplicating */
+EXTERN void
+iscsi_set_fd_dup_cb(struct iscsi_context *iscsi,
+		    void (*cb)(struct iscsi_context *iscsi, void *opaque),
+		    void *opaque);
+
 #ifdef __cplusplus
 }
 #endif
