@@ -23,10 +23,18 @@
 #define EXTERN
 #endif
 
+#include <assert.h>
+#include <stdint.h>
+#include <sys/types.h>
+
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+#ifndef static_assert
+#define static_assert(e, m) extern char unused_array[1 - 2 * !(e)]
+#endif
+
 #define SCSI_CDB_MAX_SIZE			16
 
 enum scsi_opcode {
@@ -197,6 +205,8 @@ EXTERN const char *scsi_sense_ascq_str(int ascq);
 
 EXTERN const char *scsi_pr_type_str(enum scsi_persistent_out_type pr_type);
 
+EXTERN const char *scsi_opcode_str(int opcode);
+
 enum scsi_xfer_dir {
 	SCSI_XFER_NONE  = 0,
 	SCSI_XFER_READ  = 1,
@@ -274,8 +284,10 @@ struct scsi_sense {
 	unsigned            sense_specific:1;
 	unsigned            ill_param_in_cdb:1;
 	unsigned            bit_pointer_valid:1;
+	unsigned            info_valid:1;
 	unsigned char       bit_pointer;
 	uint16_t            field_pointer;
+	uint64_t            information;
 };
 
 struct scsi_data {
@@ -527,6 +539,7 @@ enum scsi_inquiry_pagecode {
 	SCSI_INQUIRY_PAGECODE_SUPPORTED_VPD_PAGES          = 0x00,
 	SCSI_INQUIRY_PAGECODE_UNIT_SERIAL_NUMBER           = 0x80,
 	SCSI_INQUIRY_PAGECODE_DEVICE_IDENTIFICATION        = 0x83,
+	SCSI_INQUIRY_PAGECODE_THIRD_PARTY_COPY             = 0x8F,
 	SCSI_INQUIRY_PAGECODE_BLOCK_LIMITS                 = 0xB0,
 	SCSI_INQUIRY_PAGECODE_BLOCK_DEVICE_CHARACTERISTICS = 0xB1,
 	SCSI_INQUIRY_PAGECODE_LOGICAL_BLOCK_PROVISIONING   = 0xB2
@@ -678,6 +691,32 @@ struct scsi_inquiry_device_identification {
 	struct scsi_inquiry_device_designator *designators;
 };
 
+enum third_party_copy_descriptor_type {
+	THIRD_PARTY_COPY_TYPE_SUPPORTED_COMMANDS  = 0x0001
+};
+
+struct third_party_copy_command_support {
+	struct third_party_copy_command_support *next;
+
+	int operation_code;
+	int service_action_length;
+	int *service_action;
+};
+
+struct third_party_copy_supported_commands {
+	enum third_party_copy_descriptor_type descriptor_type;
+
+	struct third_party_copy_command_support *commands_supported;
+};
+
+struct scsi_inquiry_third_party_copy {
+	enum scsi_inquiry_peripheral_qualifier qualifier;
+	enum scsi_inquiry_peripheral_device_type device_type;
+	enum scsi_inquiry_pagecode pagecode;
+
+	struct third_party_copy_supported_commands *supported_commands;
+};
+
 /*
  * MODESENSE
  */
@@ -1160,6 +1199,7 @@ enum ec_descr_type_code {
 	STRM_TO_BLK_SEG_DESCR	= 0x01,
 	BLK_TO_BLK_SEG_DESCR	= 0x02,
 	STRM_TO_STRM_SEG_DESCR	= 0x03,
+	BLK_TO_BLK_OFF_SEG_DESCR = 0x0A,
 
 	/* Target descriptors : 0xEO to 0xFE */
 	IDENT_DESCR_TGT_DESCR	= 0xE4,
